const express = require('express')
const path = require('path')

const rootPath = path.resolve.bind(path, __dirname, '../')

module.exports = ({ publicPath = {}, cssUrl }) => {
  const app = express()
  app.get('/__IFRAME__', (req, res) => {
    res.send(`
      <html lang="en">
        <head>
          <meta charSet="utf-8" />
          <meta name="viewport" content="width=device-width, minimum-scale=1" />
          <link type="text/css" rel="stylesheet" href="${cssUrl}" />
          <script src="/__ASSETS__/scripts/bundle/iframe.js"></script>
        </head>
        <body>
          <div id="preview"></div>
        </body>
      </html>
    `)
  })
  Object.keys(publicPath).forEach(key => {
    publicPath[key].forEach(p => {
      app.use(key, express.static(p))
    })
  })
  if (process.env.SLDS_PREVIEWER === 'development') {
    const webpack = require('webpack')
    const webpackMiddleware = require('webpack-dev-middleware')
    const webpackConfig = require('../webpack.config')
    app.use(webpackMiddleware(webpack(webpackConfig), {
      noInfo: true,
      publicPath: webpackConfig.output.publicPath,
      quiet: false
    }))
  }
  app.use(express.static(rootPath('public')))
  app.use('/__ASSETS__', express.static(
    path.resolve(
      path.dirname(require.resolve('@salesforce-ux/design-system/package.json')),
      'assets'
    )
  ))
  return app
}
