const I = require('immutable-ext')
const Either = require('data.either')
const { applyModifiers } = require('@salesforce-ux/design-system-markup/server')
const {modifiers} = require('../../lib/finds')

const currentSelectedSectionKeyPath = state =>
  Either.fromNullable(state.get('currentSelection'))
  .map(keypath => keypath.split('-'))

const foundModifiers = (mods, item) =>
  mods.filter(m => item.get('markup').match(m.getIn(['annotations', 'selector']).replace('.', '')))

const setDefaultMods = state =>
  currentSelectedSectionKeyPath(state)
  .map(keypath =>
    foundModifiers(modifiers(state.get('component')), state.get('markup').getIn(keypath)))
  .getOrElse(I.List())

const applyModsToSelectionOrAll = (state, sections) => {
  let mods = I.Set()

  const updateMarkupAndMods = item => {
    const {markup, modifiers} = applyModifiers(state.get('modifiers'), item.get('markup'))
    mods = mods.concat(I.fromJS(modifiers)) // cheating :/
    return item.set('markup', markup)
  }

  const sects = currentSelectedSectionKeyPath(state)
  .map(keypath =>
    sections.updateIn(keypath, updateMarkupAndMods)
  )
  .orElse(() =>
    Either.of(sections.map(section =>
      section.update('items', items => items.map(updateMarkupAndMods))
    ))
  )
  .get()
  return {sections: sects, modifiers: mods}
}

module.exports = {setDefaultMods, applyModsToSelectionOrAll}
